import re
import random
import importlib
import datetime
import logging
import os

import toml
import telegram
from telegram.ext import MessageHandler, Filters
from telegram.ext import Updater
from telegram.ext import CommandHandler


class Bot:
    def __init__(self, filename):
        """Initialize a bot object

        Args:
            filename: config file for the bot
        """
        logging.info("Loading configuration from %s.", filename)
        self.config = toml.load(filename)
        self.name = self.config["bot"]['name']
        self.bot = telegram.Bot(os.environ[self.config["bot"]['token']])
        self.channels = {}
        self.reactions = {}
        for k in self.config["match"].keys():
            self.add_reaction(k, self.config["match"][k])

        self.pings = []
        for quote in self.config["ping"]["quotes"]:
            self.add_ping(quote)

        for channel in self.config["bot"]["channels"]:
            self.channels[channel] = datetime.datetime(1,1,1)


        self.updater = Updater(token=os.environ[self.config["bot"]["token"]], use_context=True)
        self.dispatcher = self.updater.dispatcher

        self.quote_handler = CommandHandler('ping', self.on_ping)
        self.dispatcher.add_handler(self.quote_handler)

    def start(self):
        self.updater.start_polling()

    def stop(self):
        self.updater.stop()

    def on_join(self):
        pass

    def on_ping(self, update, context):
        quote = random.choice(self.config["ping"]["quotes"])
        context.bot.send_message(chat_id=update.effective_chat.id, text=quote)

    def add_reaction(self, match, reaction):
        """Add a reaction to the bot.

        Args:
            match: The string which, if matched will trigger the answer.
            reaction: The string which will be sent.
        """
        self.reactions[match] = reaction

    def add_python_reaction(self, match, reaction):
        """ Add a Python callback to the reactions.

        Args:
            match: The string which, if matched will trigger the answer.
            reaction: The path to the callback
        """
        self.add_reaction(match, self.fetch_callback(reaction))

    def add_ping(self, reaction):
        """Add a reaction to a ping"""
        self.pings.append(reaction)

    def add_python_ping(self, reaction):
        """Fetch a Python callable and add it to the pings"""
        self.add_ping(self.fetch_callback(reaction))

    def fetch_callback(self, path):
        """Fetch a Python callable"""
        s = path.split(".")
        module, callback = ".".join(s[:-1]), s[-1]
        module = importlib.import_module(module)
        return getattr(module, callback)

    def get_reaction(self, username, channel, message):
        """Get a reaction to a message.

        Args:
            user: The user who sent the message.
            channel: The channel on which the bot speak.
            message: The message to which the bot has to react.

        Returns:
            Every matched reactions.
        """
        if (datetime.datetime.now() - self.last_time).total_seconds() < self.min_time:
            return []

        context = {
            "channel": channel,
            "name": self.name,
            "user": username,
            "message": message,
        }
        result = []
        for m in self.reactions.keys():
            search = m.search(message)
            if search:
                r = self.reactions[m]
                if callable(r):
                    r = r(self, username, channel, message)
                else:
                    r = r.format(**context)
                result.append(r)

        if not result and self.ping_match.search(message):
            r = random.choice(self.pings)
            if callable(r):
                r = r(self, username, channel, message)
            else:
                r = r.format(**context)
            result.append(" : ".join([username, r]))

        if len(result) > 0:
            self.last_time = datetime.datetime.now()

        return result
